#!/usr/bin/env python

from netCDF4 import Dataset
import numpy as np
import pandas as pd
import sys

# What is going on ---------------------------------------------------------------
print 'Syntax:'
print '       ./datapoint-netcdf.py [Numpy file] [year] [center OR p1s OR m1s] [leap/noleap]'
print ' '
print 'Argument List:', str(sys.argv)
print ''
#     sys.argv[0]: ', python code
#     sys.argv[1]: ', numpy file
#     sys.argv[2]: ', year
#     sys.argv[3]: ', type of simulation
#     sys.argv[4]: ', leap or no leap   
# --------------------------------------------------------------------------------

# defining constants
ndim      = 123234
tdim      = [186, 168, 186, 180, 186, 180, 186, 186, 180, 186, 180, 185]     # no leap
if sys.argv[4] == 'leap':
   tdim[1] = 174   # ADjust for February
MONTHS    = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"]
mnths_num = [ "01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12" ]

# Reading lat/lon of points
ff      = open('edges.csv','r')
header1 = ff.readline()
wlat    = np.zeros(shape=ndim)
wlon    = np.zeros(shape=ndim)

# Putting lat/lon into new array
m  = 0
for line in ff:
    liney   = line.strip()
    columns = liney.split(",")
    wlon[m] = float(columns[0])
    wlat[m] = float(columns[1])
    m = m + 1
ff.close()

# Reading wave data from numpy
swh = np.load(sys.argv[1])

# Loop over months
for mnth in range(12):
    ind_st = 0
    ind_en = 0

    if mnth != 0:
       if mnth != 1:
           for i in range(mnth):
               ind_st = tdim[i] + ind_st
           for i in range(mnth+1):
               ind_en = tdim[i] + ind_en
       else: 
           ind_st = tdim[mnth-1]
           ind_en =  tdim[0]+tdim[1]
    else:
       ind_en =  tdim[mnth]

    # Slicing the dataframe of Hs time series
    # print  ind_st    # uncomment for troubleshooting
    # print  ind_en    # uncomment for troubleshooting
    wave_t = swh[:, ind_st:ind_en] 

    # Organizing wave data
    wave = np.zeros(shape=[tdim[mnth],ndim])
    wave = np.transpose(wave_t)

    # Naming convention ------------------------------------------------------------------
    # if it is a centerline Hs, don't add the suffix
    if sys.argv[3] == 'center' :
         datawave = Dataset( sys.argv[2] + mnths_num[mnth] + '_Hs.nc', 'w', format='NETCDF4_CLASSIC')
    # if it is not a centerline, append the suffix: m1s or p1s
    else:
         datawave = Dataset( sys.argv[2] + mnths_num[mnth] + '_Hs_' + sys.argv[3] + '.nc', 'w', format='NETCDF4_CLASSIC')

    # Global attributes ------------------------------------------------------------------
    datawave.description = MONTHS[mnth] + ' 2015 significant wave height simulation data'
 
    # Type of simulation
    if sys.argv[3] == 'center':
       datawave.simulation = 'centerline'
    elif sys.argv[3] == 'p1s':
       datawave.simulation = 'One positive standard deviation'
    else:
       datawave.simulation = 'One negative standard deviation'

    # Output time interval
    datawave.time_interval = "4 hours or 14400 seconds"
  
    # dimensions ------------------------------------------------------------------------
    node = datawave.createDimension('node',ndim)
    #lat  = datawave.createDimension('lat', ndim)
    #lon  = datawave.createDimension('lon', ndim)
    time = datawave.createDimension('time', tdim[mnth])

    # variables -------------------------------------------------------------------------
    lat  = datawave.createVariable('lat', np.float64, ('node',))
    lat.long_name     = 'latitude'
    lat.units         = 'degrees_north'
    lat.standard_name = 'latitude' 
    lat.axis          = 'Y' 

    lon               = datawave.createVariable('lon', np.float64, ('node',))
    lon.long_name     = "longitude"
    lon.units         = 'degrees_east'
    lon.standard_name = 'longitude' 
    lon.axis          = 'X' 

    # time variable
    time              = datawave.createVariable('time', 'i4' , ('time',))
    time.units        = 'seconds since ' + sys.argv[2] + '-' + mnths_num[mnth] + '-01 00:00:00 UTC'             
    time.long_name    = 'time'
    time.axis         = 'T'
    time.calendar     = 'noleap'
    if sys.argv[4] == 'leap':
       time.calendar     = 'leap year'

    hs                = datawave.createVariable('hs', np.float64, ('time','node'))
    hs.units          = 'meters'
    hs.standard_name  = 'significant wave height'

    # Creating time ------------------------------------------------------------------
    dates = [ 14400 + n*14400 for n in range(hs.shape[0])]

    # data writing -------------------------------------------------------------------
    lat[:]  = wlat
    lon[:]  = wlon
    time[:] = dates
    hs[:]   = wave

    datawave.close()
    print 'The NetCDF data point file created for ' + MONTHS[mnth] + ' ' + sys.argv[2]
